@@ -7,6 +7,7 @@
 #include "proxy.h"
 #include "servercapabilities.h"
 #include "sizeformatting_base.h"
+#include "socket_errors.h"
 
 #include <libfilezilla/event_loop.hpp>
 #include <libfilezilla/iputils.hpp>
@@ -138,6 +139,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		oldOperation = std::move(operations_.back());
 		operations_.pop_back();
 
+		LogMessage(MessageType::Debug_Verbose, L"%s::Reset(%d) in state %d", oldOperation->name_, nErrorCode, oldOperation->opState);
 		nErrorCode = oldOperation->Reset(nErrorCode);
 	}
 	if (!operations_.empty()) {
@@ -434,8 +436,8 @@ int CControlSocket::CheckOverwriteFile()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-CFileTransferOpData::CFileTransferOpData(bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
-	: COpData(Command::transfer)
+CFileTransferOpData::CFileTransferOpData(wchar_t const* name, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
+	: COpData(Command::transfer, name)
 	, localFile_(local_file), remoteFile_(remote_file), remotePath_(remote_path)
 	, download_(is_download)
 	, transferSettings_(settings)
@@ -575,6 +577,7 @@ int CControlSocket::SendNextCommand()
 			return FZ_REPLY_WOULDBLOCK;
 		}
 
+		LogMessage(data.sendLogLevel_, L"%s::Send() in state %d", data.name_, data.opState);
 		int res = data.Send();
 		if (res != FZ_REPLY_CONTINUE) {
 			if (res == FZ_REPLY_OK) {
@@ -601,14 +604,14 @@ int CControlSocket::SendNextCommand()
 
 int CControlSocket::ParseSubcommandResult(int prevResult, COpData const& opData)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CControlSocket::ParseSubcommandResult(%d)", prevResult);
 	if (operations_.empty()) {
-		LogMessage(MessageType::Debug_Warning, L"ParseSubcommandResult called without active operation");
+		LogMessage(MessageType::Debug_Warning, L"CControlSocket::ParseSubcommandResult(%d) called without active operation", prevResult);
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	auto & data = *operations_.back();
+	LogMessage(MessageType::Debug_Verbose, L"%s::SubcommandResult(%d) in state %d", data.name_, prevResult, data.opState);
 	int res = data.SubcommandResult(prevResult, opData);
 	if (res == FZ_REPLY_WOULDBLOCK) {
 		return FZ_REPLY_WOULDBLOCK;
@@ -917,7 +920,7 @@ int CRealControlSocket::Send(unsigned char const* buffer, unsigned int len)
 		int written = m_pBackend->Write(buffer, len, error);
 		if (written < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket::error_description(error));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket_error_description(error));
 				LogMessage(MessageType::Error, _("Disconnected from server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
@@ -956,13 +959,13 @@ void CRealControlSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_even
 	{
 	case fz::socket_event_flag::connection_next:
 		if (error) {
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), fz::socket::error_description(error));
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), fz::socket_error_description(error));
 		}
 		SetAlive();
 		break;
 	case fz::socket_event_flag::connection:
 		if (error) {
-			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), fz::socket::error_description(error));
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), fz::socket_error_description(error));
 			OnClose(error);
 		}
 		else {
@@ -1012,7 +1015,7 @@ int CRealControlSocket::OnSend()
 		int written = m_pBackend->Write(sendBuffer_.get(), sendBuffer_.size(), error);
 		if (written < 0) {
 			if (error != EAGAIN) {
-				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket::error_description(error));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket_error_description(error));
 				if (GetCurrentCommandId() != Command::connect) {
 					LogMessage(MessageType::Error, _("Disconnected from server"));
 				}
@@ -1043,7 +1046,7 @@ void CRealControlSocket::OnClose(int error)
 			LogMessage(messageType, _("Connection closed by server"));
 		}
 		else {
-			LogMessage(messageType, _("Disconnected from server: %s"), fz::socket::error_description(error));
+			LogMessage(messageType, _("Disconnected from server: %s"), fz::socket_error_description(error));
 		}
 	}
 	DoClose();
@@ -1076,7 +1079,7 @@ int CRealControlSocket::DoConnect(std::wstring const& host, unsigned int port)
 											engine_.GetOptions().GetOption(OPTION_PROXY_PASS));
 
 		if (res != EINPROGRESS) {
-			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), fz::socket::error_description(res));
+			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), fz::socket_error_description(res));
 			return FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
 		}
 	}
@@ -1095,7 +1098,7 @@ int CRealControlSocket::DoConnect(std::wstring const& host, unsigned int port)
 
 	// Treat success same as EINPROGRESS, we wait for connect notification in any case
 	if (res && res != EINPROGRESS) {
-		LogMessage(MessageType::Error, _("Could not connect to server: %s"), fz::socket::error_description(res));
+		LogMessage(MessageType::Error, _("Could not connect to server: %s"), fz::socket_error_description(res));
 		return FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR; 
 	}
 
