@@ -9,12 +9,12 @@ class CHttpFileTransferOpData final : public CFileTransferOpData, public CHttpOp
 {
 public:
 	CHttpFileTransferOpData(CHttpControlSocket & controlSocket, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings)
-		: CFileTransferOpData(is_download, local_file, remote_file, remote_path, settings)
+		: CFileTransferOpData(L"CHttpFileTransferOpData", is_download, local_file, remote_file, remote_path, settings)
 		, CHttpOpData(controlSocket)
 	{}
 
 	virtual int Send() override;
-	virtual int ParseResponse() override;
+	virtual int ParseResponse() override { return FZ_REPLY_INTERNALERROR; }
 	virtual int SubcommandResult(int prevResult, COpData const& previousOperation) override;
 
 private:
