@@ -53,8 +53,10 @@ enum RequestId
 	reqId_interactiveLogin, // gives a challenge prompt for a password
 	reqId_hostkey,			// used only by SSH/SFTP to indicate new host key
 	reqId_hostkeyChanged,	// used only by SSH/SFTP to indicate changed host key
-	reqId_certificate		// sent after a successful TLS handshake to allow certificate
+	reqId_certificate,		// sent after a successful TLS handshake to allow certificate
 							// validation
+	reqId_insecure_ftp		// If using opportunistic FTP over TLS, ask user whether he really wants
+							// to use plaintext FTP
 };
 
 class CNotification
@@ -409,8 +411,9 @@ public:
 		std::wstring const& sessionCipher,
 		std::wstring const& sessionMac,
 		int algorithmWarnings,
-	    std::vector<CCertificate> && certificates,
-	    bool hostnameMismatch);
+		std::vector<CCertificate> && certificates,
+		bool systemTrust,
+		bool hostnameMismatch);
 	virtual RequestId GetRequestID() const { return reqId_certificate; }
 
 	std::wstring const& GetHost() const { return m_host; }
@@ -436,6 +439,7 @@ public:
 
 	int GetAlgorithmWarnings() const { return m_algorithmWarnings; }
 
+	bool SystemTrust() const { return systemTrust_; }
 	bool MismatchedHostname() const { return hostnameMismatch_; }
 
 private:
@@ -450,6 +454,7 @@ private:
 
 	std::vector<CCertificate> m_certificates;
 
+	bool systemTrust_{};
 	bool hostnameMismatch_{};
 };
 
@@ -463,4 +468,14 @@ public:
 	CLocalPath dir;
 };
 
+class CInsecureFTPNotification final : public CAsyncRequestNotification
+{
+public:
+	CInsecureFTPNotification(CServer const& server);
+	virtual RequestId GetRequestID() const { return reqId_insecure_ftp; }
+
+	CServer const server_;
+	bool allow_{};
+};
+
 #endif
