@@ -1,34 +1,38 @@
-#ifndef __CMDLINE_H__
-#define __CMDLINE_H__
+#ifndef FILEZILLA_INTERFACE_CMDLINE_HEADER
+#define FILEZILLA_INTERFACE_CMDLINE_HEADER
 
 #include <wx/cmdline.h>
 
-class CCommandLine
+class CCommandLine final
 {
 public:
 	enum t_switches
 	{
 		sitemanager,
 		close,
-		version
+		version,
+		debug_startup
 	};
 
 	enum t_option
 	{
 		logontype,
-		site
+		site,
+		local
 	};
 
 	CCommandLine(int argc, wxChar** argv);
 	bool Parse();
 	void DisplayUsage();
 
-	bool HasSwitch(enum t_switches s) const;
-	wxString GetOption(enum t_option option) const;
+	bool HasSwitch(t_switches s) const;
+	wxString GetOption(t_option option) const;
 	wxString GetParameter() const;
 
+	bool BlocksReconnectAtStartup() const;
+
 protected:
 	wxCmdLineParser m_parser;
 };
 
-#endif //__CMDLINE_H__
+#endif
